@@ -22,10 +22,10 @@ To test applications that use App Engine services, such as datastore,
 developers can use the available stub implementations. Service stubs behave like
 the original service without causing permanent side effects. The datastore stub,
 for example, allows you to write entities into memory without storing them to
-the actual datastore. The testbed module makes using those stubs for testing
+the actual datastore. The `testbed` module makes using those stubs for testing
 easier.
 
-Example::
+Example:
 
     import unittest
 
@@ -68,7 +68,8 @@ Example::
 Enable stubs and disable services
 ---------------------------------
 
-The testbed module allows you to use stubs for the following services:
+The `testbed` module allows you to use stubs for the following services:
+
     - capability_service
     - channel
     - datastore_v3 (aka datastore)
@@ -92,8 +93,8 @@ Environment variables
 
 App Engine service stubs often depend on environment variables. For example, the
 datastore stub uses an environment variable to store entities linked to a
-particular app. testbed will use default values if nothing else is provided, but
-you can change those values with `self.setup_env()`.
+particular app. `testbed` will use default values if nothing else is provided,
+but you can change those values with `self.setup_env()`.
 """
 
 
@@ -103,7 +104,9 @@ you can change those values with `self.setup_env()`.
 
 
 
+import contextvars
 import os
+import re
 import sys
 import threading
 
@@ -115,16 +118,23 @@ from google.appengine.api import datastore_file_stub
 from google.appengine.api import full_app_id
 from google.appengine.api import request_info
 from google.appengine.api import stublib
+from google.appengine.api import urlfetch_stub
 from google.appengine.api import user_service_stub
+from google.appengine.api.app_identity import app_identity
 from google.appengine.api.app_identity import app_identity_stub
+from google.appengine.api.blobstore import blobstore_stub
+from google.appengine.api.blobstore import dict_blob_storage
 from google.appengine.api.capabilities import capability_stub
+from google.appengine.api.images import images_stub
 from google.appengine.api.memcache import memcache_stub
 from google.appengine.api.modules import modules_stub
+from google.appengine.api.namespace_manager import namespace_manager
 from google.appengine.api.taskqueue import taskqueue_stub
 from google.appengine.datastore import cloud_datastore_v1_stub
 from google.appengine.datastore import datastore_pbs
 from google.appengine.datastore import datastore_stub_util
 from google.appengine.datastore import datastore_v4_stub
+from google.appengine.runtime import context
 import six
 
 
@@ -141,10 +151,6 @@ except AttributeError:
 
 
   mail_stub = None
-try:
-  from google.appengine.api.logservice import logservice_stub
-except ImportError:
-  logservice_stub = None
 
 
 
@@ -166,20 +172,10 @@ try:
 except ImportError:
   apiserver_util = None
 
-gae_runtime = os.environ.get('GAE_RUNTIME', '')
-if not gae_runtime.startswith('python3'):
-  from google.appengine.api.blobstore import blobstore_stub
-  from google.appengine.api.blobstore import dict_blob_storage
-  from google.appengine.api.images import images_stub
-
-
-  from google.appengine.api import urlfetch_stub
-
-  from cloudstorage import common as gcs_common
-  from cloudstorage import stub_dispatcher as gcs_dispatcher
-
 
 DEFAULT_ENVIRONMENT = {
+    'GAE_APPLICATION': 'testbed-test',
+    'GOOGLE_CLOUD_PROJECT': 'testbed-test',
     'AUTH_DOMAIN': 'gmail.com',
     'HTTP_HOST': 'testbed.example.com',
     'CURRENT_MODULE_ID': 'default',
@@ -196,7 +192,7 @@ DEFAULT_ENVIRONMENT = {
 
 # Deprecated legacy aliases for default environment variables. New code
 
-DEFAULT_APP_ID = 'testbed-test'
+DEFAULT_APP_ID = DEFAULT_ENVIRONMENT['GAE_APPLICATION']
 DEFAULT_AUTH_DOMAIN = DEFAULT_ENVIRONMENT['AUTH_DOMAIN']
 DEFAULT_SERVER_NAME = DEFAULT_ENVIRONMENT['SERVER_NAME']
 DEFAULT_SERVER_SOFTWARE = DEFAULT_ENVIRONMENT['SERVER_SOFTWARE']
@@ -210,7 +206,6 @@ CHANNEL_SERVICE_NAME = 'channel'
 DATASTORE_SERVICE_NAME = 'datastore_v3'
 FILES_SERVICE_NAME = 'file'
 IMAGES_SERVICE_NAME = 'images'
-LOG_SERVICE_NAME = 'logservice'
 MAIL_SERVICE_NAME = 'mail'
 MEMCACHE_SERVICE_NAME = 'memcache'
 TASKQUEUE_SERVICE_NAME = 'taskqueue'
@@ -226,7 +221,6 @@ INIT_STUB_METHOD_NAMES = {
     CAPABILITY_SERVICE_NAME: 'init_capability_stub',
     DATASTORE_SERVICE_NAME: 'init_datastore_v3_stub',
     IMAGES_SERVICE_NAME: 'init_images_stub',
-    LOG_SERVICE_NAME: 'init_logservice_stub',
     MAIL_SERVICE_NAME: 'init_mail_stub',
     MEMCACHE_SERVICE_NAME: 'init_memcache_stub',
     MODULES_SERVICE_NAME: 'init_modules_stub',
@@ -239,43 +233,13 @@ INIT_STUB_METHOD_NAMES = {
 SUPPORTED_SERVICES = sorted(INIT_STUB_METHOD_NAMES)
 
 
+URLMATCHERS_TO_FETCH_FUNCTIONS = []
+
+
 AUTO_ID_POLICY_SEQUENTIAL = datastore_stub_util.SEQUENTIAL
 AUTO_ID_POLICY_SCATTERED = datastore_stub_util.SCATTERED
 
 
-
-def urlfetch_to_gcs_stub(url, payload, method, headers, request, response,
-                         follow_redirects=False, deadline=None,
-                         validate_certificate=None, http_proxy=None):
-
-  """Forwards Google Cloud Storage `urlfetch` requests to gcs_dispatcher."""
-  headers_map = dict(
-      (header.Key.lower(), header.Value) for header in headers)
-  result = gcs_dispatcher.dispatch(method, headers_map, url, payload)
-  response.StatusCode = result.status_code
-  response.Content = six.ensure_binary(
-      result.content[:urlfetch_stub.MAX_RESPONSE_SIZE])
-  for k, v in result.headers.items():
-    if k.lower() == 'content-length' and method != 'HEAD':
-      v = len(response.Content)
-    header_proto = response.header.add()
-    header_proto.Key = k
-    header_proto.Value = str(v)
-  if len(result.content) > urlfetch_stub.MAX_RESPONSE_SIZE:
-    response.contentwastruncated = True
-
-
-def urlmatcher_for_gcs_stub(url):
-  """Determines whether a URL should be handled by the Cloud Storage stub."""
-  return url.startswith(gcs_common.local_api_url())
-
-
-
-GCS_URLMATCHERS_TO_FETCH_FUNCTIONS = [
-    (urlmatcher_for_gcs_stub, urlfetch_to_gcs_stub),
-]
-
-
 class Error(Exception):
   """Base testbed error type."""
 
@@ -330,9 +294,9 @@ class EmulatorSupportChecker(object):
   def check(cls):
     """Checks whether cloud datastore should be used.
 
-    In a unittest test process, the first call to this method sets the value of
-    _use_datastore_emulator and _api_port. Subsequent calls to this method can
-    read _use_datastore_emulator.
+    In a unit test process, the first call to this method sets the value of
+    `_use_datastore_emulator` and `_api_port`. Subsequent calls to this method
+    can read `_use_datastore_emulator`.
 
     Returns:
       A boolean that indicates whether cloud datastore should be used.
@@ -383,6 +347,7 @@ class Testbed(object):
     self._enabled_stubs = {}
 
     self._blob_storage = None
+    self._context_reset_tokens = {}
 
   def activate(self, use_datastore_emulator=False):
     """Activates the testbed.
@@ -392,9 +357,11 @@ class Testbed(object):
     the application ID. You can set custom values with `setup_env()`.
 
     Args:
-      use_datastore_emulator: True if user specifies testbed to use the Cloud
+      use_datastore_emulator: `True` if user specifies testbed to use the Cloud
         Datastore Emulator.
     """
+    namespace_manager._TESTBED_RESET_TOKEN = None
+    self._context_reset_tokens = {}
     self._orig_env = dict(os.environ)
     self.setup_env()
 
@@ -451,6 +418,28 @@ class Testbed(object):
 
     os.environ.clear()
     os.environ.update(self._orig_env)
+
+    all_reset_tokens = {
+        **self._context_reset_tokens,
+        namespace_manager._CURRENT_NAMESPACE:
+            namespace_manager._TESTBED_RESET_TOKEN,
+
+    }
+
+    for ctxvar, token in all_reset_tokens.items():
+      if token:
+        try:
+          ctxvar.reset(token)
+        except ValueError:
+
+
+
+
+          pass
+
+    namespace_manager._TESTBED_RESET_TOKEN = None
+
+
     self._blob_storage = None
     self._activated = False
 
@@ -458,14 +447,14 @@ class Testbed(object):
     """Sets default and custom environment variables.
 
     By default, all of the items in `DEFAULT_ENVIRONMENT` will be created
-    without being specified.  To set a value other than the default, or to pass
+    without being specified. To set a value other than the default, or to pass
     a custom environment variable, pass a corresponding keyword argument.
 
-    Example::
+    Example:
 
         # All defaults
         testbed_instance.setup_env()
-        # All defaults, overriding AUTH_DOMAIN
+        # All defaults, overriding AUTH_DOMAIN in both context modes
         testbed_instance.setup_env(auth_domain='custom')
         # All defaults; adds a custom os.environ['CUSTOM'] = 'foo'
         testbed_instance.setup_env(custom='foo')
@@ -474,38 +463,99 @@ class Testbed(object):
     To overwrite the values set by a previous invocation, pass `overwrite=True`.
     Passing this value will not result in an `OVERWRITE` entry in `os.environ`.
 
+    If the variable corresponds to a WSGI environ variable, it will be set in
+    the gen2 request context as contextvars as well as `os.environ`. To only set
+    the gen2 context, switch to `setup_wsgi_env()`.
+
     Args:
-      overwrite: Boolean; specifies whether to overwrite items with
+      overwrite: Boolean. Specifies whether to overwrite items with
           corresponding entries in `os.environ`.
       **kwargs: Environment variables to set. The name of the argument will be
           uppercased and used as a key in `os.environ`.
     """
-    merged_kwargs = {key.upper(): value for key, value in kwargs.items()}
+    user_specified = {key.upper(): value for key, value in kwargs.items()}
 
 
 
 
 
-    specified_app_id = merged_kwargs.get('APP_ID',
-                                         full_app_id.get(environ=merged_kwargs))
-    full_app_id.clear(environ=merged_kwargs)
-    merged_kwargs.pop('APP_ID', None)
-    if full_app_id.get():
 
 
-      if specified_app_id and overwrite:
-        full_app_id.put(specified_app_id)
-    else:
 
-      full_app_id.put(specified_app_id or DEFAULT_APP_ID)
+    specified_app_id = (user_specified.get('APP_ID')
+                        or full_app_id.get(environ=user_specified)
+                        or user_specified.get('GOOGLE_CLOUD_PROJECT'))
+    full_app_id.clear(environ=user_specified)
+    user_specified.pop('APP_ID', None)
+    if specified_app_id:
 
+      user_specified.setdefault('GAE_APPLICATION', specified_app_id)
+
+
+      user_specified.setdefault('GOOGLE_CLOUD_PROJECT',
+                                full_app_id.project_id(specified_app_id))
+
+    merged_vars = user_specified.copy()
     if not overwrite:
       for key, value in six.iteritems(DEFAULT_ENVIRONMENT):
-        if key not in merged_kwargs:
-          merged_kwargs[key] = value
-    for key, value in six.iteritems(merged_kwargs):
-      if overwrite or key not in os.environ:
+        if key not in merged_vars:
+          merged_vars[key] = value
+
+    ctx = contextvars.copy_context()
+    gae_vars = {k: v for k, v in vars(context.gae_headers).items()
+                if isinstance(v, contextvars.ContextVar)}
+    wsgi_vars = {k: v for k, v in vars(context.wsgi).items()
+                 if isinstance(v, contextvars.ContextVar)}
+    for key, value in six.iteritems(merged_vars):
+      if key == 'GAE_APPLICATION':
+        if overwrite or not full_app_id.get():
+          full_app_id.put(value)
+      elif overwrite or key not in os.environ:
+        if key == 'GOOGLE_CLOUD_PROJECT':
+          validate_project_id(value)
         os.environ[key] = value
+        if key.startswith(context.gae_headers.PREFIX):
+
+          key = key[len(context.gae_headers.PREFIX):]
+        ctxvar = gae_vars.get(key, wsgi_vars.get(key))
+        if ctxvar is not None and (overwrite or ctxvar not in ctx):
+          wsgi_key = (context.gae_headers.PREFIX + key
+                      if key in gae_vars
+                      else key)
+          self.setup_wsgi_env(**{wsgi_key: value})
+
+
+
+
+  def setup_wsgi_env(self, **kwargs):
+    """Sets the gen2 request context with the WSGI vars provided in kwargs.
+
+    Valid values can be found in `google.appengine.runtime.context.gae_headers`
+    and `google.appengine.runtime.context.wsgi`.
+
+    setup_wsgi_env does not also set the "legacy context mode" aka `os.environ`.
+    If you still rely on the legacy context, use setup_env instead.
+
+    Example:
+      # Sets context.gae_headers.AUTH_DOMAIN
+      # Note that all as per PEP 3333, all headers have the HTTP_ prefix.
+      # GAE headers headers share an additional x_appengine_ prefix.
+      testbed_instance.setup_wsgi_env(http_x_appengine_auth_domain='custom')
+      # Sets context.wsgi.SERVER_NAME
+      testbed_instance.setup_wsgi_env(server_name='custom')
+
+    Args:
+      **kwargs: WSGI variables to set in the SDK's request context.
+    """
+    wsgi_env = {key.upper(): value for key, value in kwargs.items()}
+    new_reset_tokens = context.init_from_wsgi_environ(wsgi_env).items()
+
+
+
+
+    for key, value in new_reset_tokens:
+      if key not in self._context_reset_tokens:
+        self._context_reset_tokens[key] = value
 
   def _register_stub(self, service_name, stub, deactivate_callback=None):
     """Registers a service stub.
@@ -651,7 +701,7 @@ class Testbed(object):
     Args:
       enable: `True` if the fake service should be enabled, or `False` if the
           real service should be disabled.
-      datastore_file: File name of a dev_appserver datastore file.
+      datastore_file: File name of a `dev_appserver` datastore file.
       use_sqlite: `True` to use the Sqlite stub, or `False` (default) to use
           the file stub.
       auto_id_policy: How datastore stub assigns auto IDs. This value can be
@@ -659,7 +709,7 @@ class Testbed(object):
       **stub_kw_args: Keyword arguments passed on to the service stub.
 
     Raises:
-      StubNotSupportedError: If datastore_sqlite_stub is None.
+      StubNotSupportedError: If `datastore_sqlite_stub` is `None`.
     """
     if self._use_datastore_emulator:
 
@@ -746,7 +796,7 @@ class Testbed(object):
   def init_images_stub(self, enable=True, **stub_kwargs):
     """Enables the images stub.
 
-    The images service stub is only available in dev_appserver because it uses
+    The images service stub is only available in `dev_appserver` because it uses
     the PIL library.
 
     Args:
@@ -760,29 +810,10 @@ class Testbed(object):
     stub = images_stub.ImagesServiceStub(**stub_kwargs)
     self._register_stub(IMAGES_SERVICE_NAME, stub)
 
-  def init_logservice_stub(self, enable=True):
-    """Enables the log service stub.
-
-    Args:
-      enable: `True` if the fake service should be enabled, or `False` if the
-          real service should be disabled.
-
-    Raises:
-      StubNotSupportedError: The logservice stub is unvailable.
-    """
-    if not enable:
-      self._disable_stub(LOG_SERVICE_NAME)
-      return
-    if logservice_stub is None:
-      raise StubNotSupportedError(
-          'The logservice stub is not supported in production.')
-    stub = logservice_stub.LogServiceStub()
-    self._register_stub(LOG_SERVICE_NAME, stub)
-
   def init_mail_stub(self, enable=True, **stub_kw_args):
     """Enables the mail stub.
 
-    The email service stub is only available in dev_appserver because it uses
+    The email service stub is only available in `dev_appserver` because it uses
     the `subprocess` module.
 
     Args:
@@ -834,17 +865,17 @@ class Testbed(object):
     self._register_stub(TASKQUEUE_SERVICE_NAME, stub)
 
   def init_urlfetch_stub(self, enable=True, urlmatchers=None):
-    """Enables the urlfetch stub.
+    """Enables the `urlfetch` stub.
 
-    The urlfetch service stub uses the urllib module to make requests. On
-    appserver, urllib also relies the urlfetch infrastructure, so using this
+    The `urlfetch` service stub uses the `urllib` module to make requests. On
+    appserver, `urllib` also relies the `urlfetch` infrastructure, so using this
     stub will have no effect.
 
     Args:
       enable: `True` if the fake service should be enabled, or `False` if the
           real service should be disabled.
-      urlmatchers: optional initial sequence of (matcher, fetcher) pairs to
-          populate urlmatchers_to_fetch_functions; matchers passed here, if
+      urlmatchers: optional initial sequence of `(matcher, fetcher)` pairs to
+          populate `urlmatchers_to_fetch_functions`; matchers passed here, if
           any, take precedence over default matchers dispatching GCS access.
     """
     if not enable:
@@ -853,8 +884,7 @@ class Testbed(object):
     urlmatchers_to_fetch_functions = []
     if urlmatchers:
       urlmatchers_to_fetch_functions.extend(urlmatchers)
-    urlmatchers_to_fetch_functions.extend(
-        GCS_URLMATCHERS_TO_FETCH_FUNCTIONS)
+    urlmatchers_to_fetch_functions.extend(URLMATCHERS_TO_FETCH_FUNCTIONS)
     stub = urlfetch_stub.URLFetchServiceStub(
         urlmatchers_to_fetch_functions=urlmatchers_to_fetch_functions)
     self._register_stub(URLFETCH_SERVICE_NAME, stub)
@@ -935,3 +965,48 @@ class Testbed(object):
     """
     for service_name in SUPPORTED_SERVICES:
       self._init_stub(service_name, enable)
+
+_PROJECT_ID_VALID_CHARS = re.compile(r'[a-z0-9\-]+')
+
+
+def validate_project_id(project_id):
+  """Ensure that a GCP project ID is valid.
+
+  From
+  https://cloud.google.com/resource-manager/docs/creating-managing-projects#before_you_begin
+  The project ID must be a unique string of 6 to 30 lowercase letters, digits,
+  or hyphens. It must start with a letter, and cannot have a trailing hyphen.
+
+  Args:
+    project_id (str): project id, potentially including domain prefix, to
+      validate.
+  """
+
+  def err(msg):
+    raise ValueError(f'Invalid GCP project ID "{project_id}": {msg}.')
+
+  if '~' in project_id:
+    err('partition prefex (e.g. "s~") not allowed in project ID '
+        '(consider gae_application instead)')
+
+  chunks = project_id.split(':')
+  if len(chunks) > 2:
+    err('at most one colon is allowed (for domain prefix)')
+
+
+  project_id = chunks[-1]
+
+  if len(project_id) < 6:
+    err('must be at least 6 characters')
+
+  if len(project_id) > 30:
+    err('must not be longer than 30 characters')
+
+  if not _PROJECT_ID_VALID_CHARS.fullmatch(project_id):
+    err('only lowercase letters, digits, or hyphens are allowed')
+
+  if not project_id[0].islower():
+    err('must start with a lowercase letter')
+
+  if project_id[-1] == '-':
+    err('must not end with a hyphen')
